﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace TreeExample
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void quitMenu_Click(object sender, EventArgs e)
        {
            Close ();
        }

        private void ShowDir (TreeNode target, 
                              DirectoryInfo dir, 
                              int level = 3)
        {
            TreeNode node = new TreeNode();
            node.Text = dir.Name;
            node.ToolTipText = dir.FullName;

            if (target == null)
               tree.Nodes.Add(node);
            else
               target.Nodes.Add (node);

            if (level > 1)
               foreach (DirectoryInfo s in dir.GetDirectories ())
                  ShowDir (node, s, level-1);
        }

        private void testMenu_Click(object sender, EventArgs e)
        {
            // using System.IO;
            DirectoryInfo d = new DirectoryInfo ("C:\\Work");
            ShowDir (null, d);

            /*
            TreeNode root = new TreeNode ();
            root.Text = "root";
            // root.BackColor = Color.Blue;

            TreeNode node = new TreeNode();
            node.Text = "subitem";
            node.ToolTipText = "nejaky text";

            root.Nodes.Add (node);

            tree.Nodes.Add (root);
            tree.ShowNodeToolTips = true;

            root.Expand ();
            */
        }

        private void dragAndDropButton_MouseDown(object sender, MouseEventArgs e)
        {
            DragDropEffects effect =
               DragDropEffects.Move |
               DragDropEffects.Copy |
               DragDropEffects.Link ;

            source = null;

            // MouseDown
            DoDragDrop ("nejaka data", effect);
        }

        TreeNode source = null;

        private void tree_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {

                DragDropEffects effect =
                   DragDropEffects.Move |
                   DragDropEffects.Copy |
                   DragDropEffects.Link;

                source = tree.GetNodeAt (e.X, e.Y);
                // source = tree.SelectedNode;

                if (source != null)
                   DoDragDrop("jina data", effect);
            }
        }

        private void textBox_DragEnter(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.All;
        }

        private void textBox_DragDrop(object sender, DragEventArgs e)
        {
           foreach (string f in e.Data.GetFormats ())
              textBox.AppendText ("format "+ f + "\n");
           textBox.AppendText (e.Data.GetData (DataFormats.Text) + "\n");
        }

        private void tree_DragEnter(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.All;
        }

        private void tree_DragOver(object sender, DragEventArgs e)
        {
            if (Control.ModifierKeys == Keys.Shift)
                e.Effect = DragDropEffects.Copy;
            else if (Control.ModifierKeys == Keys.Control)
                e.Effect = DragDropEffects.Link;
            else
                e.Effect = DragDropEffects.Move;

            Point p1 = new Point (e.X, e.Y);
            Point p2 = tree.PointToClient (p1);
            TreeNode node = tree.GetNodeAt (p2);
            tree.SelectedNode = node;

            if (e.Effect == DragDropEffects.Move && source != null)
            {
                bool ok = true;
                TreeNode p = node;
                while (p != null && ok)
                {
                    if (p == source) ok = false;
                    p = p.Parent;
                }
                if (!ok) e.Effect = DragDropEffects.None;
            }

        }

        private TreeNode CopyTree (TreeNode from)
        {
            TreeNode result = new TreeNode();
            result.Text = from.Text;

            foreach (TreeNode f in from.Nodes)
            {
                TreeNode r = CopyTree (f);
                result.Nodes.Add (r);
            }

            return result;
        }

        private void tree_DragDrop(object sender, DragEventArgs e)
        {
           TreeNode target = tree.SelectedNode;
           if (source != null)
           {
               if (e.Effect == DragDropEffects.Copy)
               {
                   TreeNode node = CopyTree (source);

                   if (target == null)
                       tree.Nodes.Add(node);
                   else
                   {
                       target.Nodes.Insert(0, node);
                       target.Expand();
                   }
               }
               else if (e.Effect == DragDropEffects.Move)
               {
                   bool ok = true;
                   TreeNode p = target;
                   while (p != null && ok)
                   {
                       if (p == source) ok = false;
                       p = p.Parent;
                   }

                   if (ok)
                   {
                       source.Remove();

                       if (target == null)
                           tree.Nodes.Add(source);
                       else
                       {
                           target.Nodes.Insert(0, source);
                           target.Expand();
                       }
                   }
               }
           }
           else
           {
               TreeNode node = new TreeNode();
               node.Text = "new item";

               if (e.Effect == DragDropEffects.Link)
               {
                   if (target == null)
                       tree.Nodes.Insert(0, node);
                   else
                   {
                       TreeNode p = target.Parent;
                       if (p != null)
                           p.Nodes.Insert(p.Nodes.IndexOf(target), node);
                       else
                           tree.Nodes.Insert(tree.Nodes.IndexOf(target), node);
                   }
               }
               else
               {
                   if (target == null)
                       tree.Nodes.Add(node);
                   else
                   {
                       target.Nodes.Insert(0, node);
                       target.Expand();
                   }
               }
           }
        }


    }
}
